//  Licensed under the Apache License, Version 2.0 (the "License"); you may not
//  use this file except in compliance with the License. You may obtain a copy of
//  the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
//  License for the specific language governing permissions and limitations under
//  the License.
//
//  Networking.m
//  iThemeFramework
//
//  Created by Chris Davis on 09/02/2013.
//  Copyright (c) 2013 QuantumLogic LTD. All rights reserved.
//

#import "Networking.h"
#import "Constants.h"

@implementation Networking

- (NSDictionary *)postImage:(NSString *)url
				  imageData:(NSData *)imageData
					   auth:(NSString *)authToken
					  error:(NSError **)error;
{
	NSURL *nsurl = [NSURL URLWithString:url];
	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:nsurl cachePolicy:NSURLRequestReloadIgnoringLocalAndRemoteCacheData timeoutInterval:60];
	[request setHTTPMethod:kHTTP_POST];
	[request setValue:authToken forHTTPHeaderField:kHTTP_HEADER_AUTHORISATION];
	
	NSString *boundary = @"0xKhTmLbOuNdArY";
	
	NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
	
	[request addValue:contentType forHTTPHeaderField: @"Content-Type"];
	
	NSMutableData *body = [NSMutableData data];
	[body appendData:[[NSString stringWithFormat:@"--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[@"Content-Disposition: form-data; name=\"file\"; filename=\"image.jpg\"\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[@"Content-Type: image/jpeg\r\n\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:imageData];
	[body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	[request setHTTPBody:body];
	
	NSHTTPURLResponse* urlResponse = nil;
	NSData *data = [NSURLConnection sendSynchronousRequest:request returningResponse:&urlResponse error:error];
	
	if (*error != nil)
	{
		DLog([*error localizedDescription]);
	}
	
	long statusCode = [urlResponse statusCode];
	if (statusCode != kHTTP_GOOD)
		return nil;
	
	return (NSDictionary *)[data objectFromJSONData];
}


- (NSDictionary *)httpSynchronousUrlWithMethod:(NSString *)url
										method:(NSString *)method
									 keyvalues:(NSDictionary *)keyvalues
										  auth:(NSString *)authToken
										 error:(NSError **)error
{
	NSString *jsonString = nil;
	NSData *jsonData = nil;
	if (keyvalues != nil)
	{
		jsonString = [keyvalues JSONString];
		jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
	}
	
	NSURL *nsurl = [NSURL URLWithString:url];
	NSMutableURLRequest *theRequest = [NSMutableURLRequest requestWithURL:nsurl];
	
	[theRequest setValue:authToken forHTTPHeaderField:kHTTP_HEADER_AUTHORISATION];
	[theRequest setHTTPMethod:method];
	
	if (keyvalues != nil)
	{
		[theRequest setValue:jsonString forHTTPHeaderField:kHTTP_TYPE_JSON];
		[theRequest setHTTPBody:jsonData];
	}
	
	NSHTTPURLResponse* urlResponse = nil;
	NSData *data = [NSURLConnection sendSynchronousRequest:theRequest returningResponse:&urlResponse error:error];
	
	if (*error != nil)
	{
		DLog([*error localizedDescription]);
	}
	
	long statusCode = [urlResponse statusCode];
	if (statusCode != kHTTP_GOOD)
		return nil;
	
	return (NSDictionary *)[data objectFromJSONData];
}
@end
