//  Licensed under the Apache License, Version 2.0 (the "License"); you may not
//  use this file except in compliance with the License. You may obtain a copy of
//  the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
//  License for the specific language governing permissions and limitations under
//  the License.
//
//  ThemeFramework.h
//  iThemeFramework
//
//  Created by Chris Davis on 05/04/2012.
//  Copyright (c) 2012 GameWeaver LTD. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <objc/runtime.h>
#import <objc/message.h>

typedef void (^StartBlock)(id self);
typedef void (^EndBlock)(id, NSDictionary *, NSError *);

@interface ThemeFramework : NSObject
{
	
}
@property (nonatomic, retain) NSMutableDictionary *ImageLookup;
@property (nonatomic, retain) UIViewController *Controller;
@property (nonatomic, retain) NSString *AppKey;
@property (nonatomic, retain) NSString *ThemeKey;
@property (nonatomic, retain) NSString *Username;
@property (nonatomic, retain) NSString *Password;
@property (nonatomic, retain) NSString *AuthToken;
@property (nonatomic, retain) NSString *PlayerId;

#pragma mark - General Init/Singleton

+ (ThemeFramework *)instance;
- (id)init;
- (id)init:(NSString *)appKey
  themeKey:(NSString *)themeKey
  username:(NSString *)username
  password:(NSString *)password;

#pragma mark - 'Public' methods

// The ensure methods ensure that the layout has been synced to
// the itheme API

- (void)ensureLayout:(UIViewController *)controller;
- (void)ensureLayout:(UIViewController *)controller
toInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation;

#pragma mark - 'Private' methods


// Called every x seconds to update the layout from the server
- (void)updateLayoutEvent;

// Syncs layout to server
- (void)ensureScreen:(UIViewController *)controller
			screenId:(NSString *)screenId
				size:(CGSize)size;

// Recursively walks the control heirarchy to generate the
// layout
- (NSArray *)walkControls:(UIView *)view;

// Recursively walks the control heirarchy to apply the
// layout
- (NSArray *)applyLayoutTo:(UIView *)view
					  with:(NSDictionary *)dic;

// Computes the screen size based on rotation.
- (CGSize)computeScreenSize;

// Computes a unique screen name for the controller.
- (NSString *)computeScreenName:(UIViewController *)controller
						   size:(CGSize)size;

// TODO: Take a screenshot of the current controller.view and
// upload.
- (void)takeAndUploadScreenshot:(UIViewController *)controller;

// TODO: Take a screenshot
- (UIImage *)takeScreenshot;

// Calls other swizzling methods
- (void)swizzleMethods;

// Swizzles a class methods to an instance method
- (void)swizzleClassMethodToInstanceMethod:(Class)c
								  instance:(id)instance
									  orig:(SEL)orig
								 newmethod:(SEL)newmethod;

// A swizzled instance method of imageNamed
- (UIImage *)_imageNamed:(NSString *)name;

// Unused, Will download an image and cache it.
- (void)downloadAndCacheImage:(NSString *)urlString;

// Takes a NSString and converts it to a UIColor
- (UIColor *)colorFromHexString:(NSString *)hexString;

// Logs the user in.
- (void)login:(NSDictionary *)playerDict
	   caller:(id)caller
		start:(StartBlock)start
		  end:(EndBlock)end;

// Converts a UIColor to a NSString
- (NSString*)colorToWeb:(UIColor*)color;
@end
